
//						
// INTRODUCTION
//    This is a simple TCP socket program for the Lambda Genesys power supply with LAN.
//    It opens a console window with a prompt to type and send SCPI commands and queries.
//		Author:   Walter Heyck, TDK-Lambda, 

//      Version: "GEN TCP Socket C# Terminal v01"	 Sept 26, 2008
//		Initial release.

//      Version: "GEN TCP Socket C# Terminal v02"    Oct 6, 2008
//      Adds "Enter IP addr: " so EXE can be run on other computers.

//      Version: "GEN TCP Socket C# Terminal v03"    Nov 26, 2008
//      Shows actual length of response string in quotes, not 255 chars always.

//      Version: "GEN TCP Socket C# Terminal v04"    Dec 2, 2008
//      Makes the socket BLOCKING so that bad queries, which return a TIMEOUT error,
//		  do not cascade into errors for subsequent queries.

//
// SETUP:
//    This program requires the Visual Studio 2005 Edit/Debug IDE.
//    Then click RUN (or F5 key).
// USAGE:
//    You will see a console window open with a prompt to enter an IP address.
//      and then see a prompt to enter a SCPI command.
//    At the command line, type in SCPI command<Enter>.
//    If the command has a "?", the program will read back a response and display it.
//    Type command "x" <Enter> to close socket and exit the program.
// NOTE:
//	  This program opens TCP socket port 8003.
//

using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.Net.Sockets;
using System.Threading;

namespace TCPSocketCSCmdLine
{
	class Program
	{
		const int targetPort = 8003;		// 8003 = port for TCP socket connection
		// Timeout value required of program locks up if there
		//    is never a response to read.
		const int RCV_TIME_OUT = 3000;	// in milliseconds

		static void Main( string[ ] args )
		{
			string cmdLine;							// SCPI command from console input
			byte[ ] recvBytes = new byte[ 256 ];	// Buffer to receive socket responses
			int recvCount = 0;						// Bytes received during socket Receive
			string sExit = "X";						// Comparitor for exiting loop

			Console.WriteLine( "\nGenesys Power Supply - TCP socket console program v04." );
			Console.WriteLine( "-- Adds socket BLOCKING = TRUE immediately before mySocket.Receive." );
			Console.WriteLine( "--     Use command \"x\" to exit --\n\n" );
			Console.WriteLine( "Enter IP address" );
			Console.Write    ( "(default 169.254.252.145) --> " );
			cmdLine = Console.ReadLine( );

			if ( cmdLine.Length < 7 )
				cmdLine = "169.254.252.145";

			IPAddress targetIP = IPAddress.Parse( cmdLine );

			Socket mySocket = new Socket( AddressFamily.InterNetwork,
											SocketType.Stream,
											ProtocolType.Tcp );

			mySocket.ReceiveTimeout = RCV_TIME_OUT;
			// DontFragment required or first response packet ends up 
			//    as response for second query
			mySocket.DontFragment = true;

			// Open connection to power supply socket
			try
			{
				mySocket.Connect( new IPEndPoint( targetIP, targetPort ) );
			}
			catch ( SocketException e )
			{
				Console.WriteLine( "\"Connect error: \" {0} Error code: {1}.", 
												e.Message, e.ErrorCode );
				throw;
			}

			// -----------------------------------------------------------------
			// Endless Loop, input and send SCPI commands until "x" is entered
			while ( true)
			{
				try
				{
					recvCount = 0;
					Array.Clear( recvBytes, 0, 256 );

					Console.Write( "\n\nEnter SCPI cmd: " );
					cmdLine = Console.ReadLine( );

					// Check if quitting program by typing "x" or "X"
					if ( string.Equals( cmdLine.ToUpper( ), sExit))
						break; 
					// Convert the string into a byte[] array for "send" method.
					byte[ ] cmdBytes = Encoding.ASCII.GetBytes( cmdLine);

					// Send TCP packet. Block this program execution until Send returns.
					int i = mySocket.Send( cmdBytes,
										cmdBytes.Length, SocketFlags.None);

					Console.WriteLine( "Sent {0} bytes.", i );

					// If SCPI string has "?", then is query and need to read response
					if ( cmdLine.IndexOf( "?" ) >= 0 )
					{
						// Added to fix problem where a bad query would timeout and the 
						//   next query would return another error 10035 and all queries there after
						//   would respond with the message from the query before it

						mySocket.Blocking = true;

						// Is query, so read the response
						recvCount = mySocket.Receive( recvBytes, SocketFlags.None );
						

						Console.Write( "Recv {0} chars = \"{1}\"", recvCount,
								Encoding.ASCII.GetString( recvBytes).Substring(0, recvCount));
					}  // End If
				}
			    catch (SocketException e)
				{
					Console.WriteLine("ERROR: \n{0} Error code: {1}.", e.Message, e.ErrorCode);
				}
			}		// End While loop

			mySocket.Close( );
			
			// Hold window open you you can examine it.
			Console.WriteLine( "\nPress <Enter> to close console and quit." );
			cmdLine = Console.ReadLine( );
		}
	}
}
