/*********************************FILE HEADER***********************************
 *                      .File:         . UDPSocketClient.c
 *                      .Project Code: . LLC(Lambda LAN Card)
 *                      .Author:       . Dinesh
 *                      .Environment   . LinuxC
 *                      .Created on:   . 11-Dec-2007
 *                      .Copyright:    . Lambda
 *
 *******************************************************************************
 *******************************************************************************
 * .Type:               . Source File
 * .Description:        . LLC client program in Linux
 * .Code Status:        . Tested
 * .Reference:          . LLC_UDPClientServer_DetailDesign.doc
 * .Revision History:

*******************************************************************************/

#include<stdio.h>
#include<sys/socket.h>
#include<sys/types.h>
#include<netinet/in.h>
#include<stdlib.h>
#include<sys/time.h>
#include<unistd.h>
#include<netdb.h>
#include<string.h>
#include<stdbool.h>

#define PORT  			    8005
#define MAX_CMD_BUF 		255
#define MAX_IP_LENGTH 		20
#define FIELD_LENGTH 		5
#define FIELD_LIM_HIGH 		255
#define FIELD_LIM_LOW 		0
#define NO_OF_FIELDS 		4
#define NO_OF_DOTS 		    3
#define TIME_OUT_LIMIT 		10
#define EXIT_STRING 		"EXIT"
#define QUESTION_MARK 		'?'
#define END_OF_STRING 		'\0'
#define DOT 			    '.'

void CloseSocket(int);      // For closing socket identifier

bool CheckIP(char []);      // For checking the format of an IP address

int main()
{
    int nTimeVar		= 0;
    int nSockDesc 		= 0;
    int nSize 		    = 0;
    bool  blCheck 		= false;
    char szIPaddress[MAX_IP_LENGTH] 	= {0};
    char szCmdBuff[MAX_CMD_BUF] 		= {0};
    char szRecvBuffer[MAX_CMD_BUF] 		= {0};
    struct sockaddr_in servAddr 		= {0};
    struct timeval timeOutVar 		    = {0};
    struct hostent *hostData 	       	= NULL;
	
    timeOutVar.tv_sec = TIME_OUT_LIMIT;
    printf("Please enter the IP address or Hostname of power supply : ");
    scanf("%s",szIPaddress);
    bzero((char*)&servAddr, sizeof(servAddr));
    //To check if it is IP Addres or Hostname
    if(isalpha(szIPaddress[0]))
    {
        hostData = gethostbyname(szIPaddress);
		if(NULL == hostData)
        {
            herror("\nHost Error");
            exit(1);
        }
    	// Filling server address
        servAddr.sin_addr = *(struct in_addr*)(hostData->h_addr);
	}
    else
    {
        // To check if IP Address is in valid format
        blCheck=CheckIP(szIPaddress);
        if(false == blCheck)
        {
            printf("\nError in IP format");
            exit(1);
        }
        // Filling server address
    	inet_pton(AF_INET, szIPaddress, &servAddr.sin_addr);
    }
    //Read out the data in keyboard buffer
	getchar();
    while(1)
    {
        memset(szCmdBuff, 0, MAX_CMD_BUF);
        printf("\nEnter SCPI command or Type EXIT : ");
        fflush(stdin);
        fgets(szCmdBuff, MAX_CMD_BUF, stdin);
        //check if the user entered EXIT
        if (0 == strncmp(szCmdBuff, EXIT_STRING, strlen(EXIT_STRING)))
        {
            exit(1);
        }
        // UDP socket creation
        nSockDesc=socket(AF_INET, SOCK_DGRAM, 0);
        if(0 > nSockDesc)
        {
            perror("\nSocket creation failed");
            exit(1);
        }
        // Timeout checking
        nTimeVar = setsockopt(nSockDesc, SOL_SOCKET, SO_RCVTIMEO,
               			           (struct timeval *)&timeOutVar, 
				                         sizeof(struct timeval));
        if(0 > nTimeVar)
        {
            perror("\nSocket option setting failed");
            CloseSocket(nSockDesc);
            exit(1);
        }
        // Filling server details
        servAddr.sin_family=AF_INET;
        servAddr.sin_port=htons(PORT);
        // Sending command by eleminating new line character
        nSize=sendto(nSockDesc, szCmdBuff, strlen(szCmdBuff)-1, 0,
                   				      (struct sockaddr*)&servAddr,
						                        sizeof(servAddr));
        if(0 > nSize)
        {
            perror("\nCould not send");
            CloseSocket(nSockDesc);
            continue;
        }
        // Receive only if '?' exists at the end of the SCPI command
        if(QUESTION_MARK == szCmdBuff[nSize-1])
        {
            // clear the contents of Receive Buffer
            memset(szRecvBuffer, 0, MAX_CMD_BUF);
            // receiving
            nSize=recvfrom(nSockDesc, szRecvBuffer, MAX_CMD_BUF, 0, NULL, NULL);
            if(0 > nSize)
            {
                perror("\nCould not receive");
                CloseSocket(nSockDesc);
                continue;
            }
            printf("\nResponse : %s",szRecvBuffer);
        }
        CloseSocket(nSockDesc);
        printf("\nSuccess\n");
    }// End while
    return 0;
}


/************************FUNCTION HEADER****************************************
*       Function      : CloseSocket
*       Description   : Function for closing socket
*       Return Value  : void
*       Called by     :
*       Remarks
*******************************************************************************/

void  CloseSocket(int nSockID)
{
    if(0 != close(nSockID))
    {
        printf("\nCould not close");
        exit(1);
    }
}

/************************FUNCTION HEADER****************************************
*       Function      : CheckIP
*       Description   : Function for checking format of IP Address
*       Return Value  : true if True else false
*       Called by     :
*       Remarks
*******************************************************************************/

bool CheckIP(char szAddress[])
{
    int nFieldNum 		= 0;
    int nCnt 		    = 0;
    int nFieldCount 	= 0;
    int nLength 		= 0;
    int nDotsNum 		= 0;
	char szFieldArray[FIELD_LENGTH] = {0};

    // For checking the number of dots in IP Address
    for(nCnt = 0; nCnt < strlen(szAddress); nCnt++)
    {
        if(DOT == szAddress[nCnt])
        {
            nDotsNum++;
        }
    }
    if( NO_OF_DOTS != nDotsNum )
    {
        return false;
    }
    // Checking each field in the IP Address with in the limit
    while(NO_OF_FIELDS > nFieldCount)
    {
        nCnt=0;
        // Extracting each field by comparing with '.'
        while(szAddress[nLength]!= DOT && nLength < strlen(szAddress))
        {
            if(isdigit(szAddress[nLength]))
            {
                szFieldArray[nCnt] = szAddress[nLength];
                nCnt++;
                nLength++;
            }
            else
            {
                return false;
            }
        }
        szFieldArray[nCnt] = END_OF_STRING;
        nFieldNum = atoi(szFieldArray);
        // Checking the boundary limits of each field
        if((FIELD_LIM_LOW <= nFieldNum) && (nFieldNum <= FIELD_LIM_HIGH))
        {
            nLength++;
            nFieldCount++;
        }
        else
        {
            return false;
        }
    }
    return true;
}

