/*********************************.FILE_HEADER.********************************
 *            .File:         . MainForm.cs
 *            .Project Code: . LLC
 *            .Author:       . Shijoy.J.G
 *            .Environment:  . User Mode
 *            .Created on:   . 14-01-2008
 *            .Copyright:    . 2005 Lambda Americas.
 *
 ******************************************************************************
 * .Type:             .Source File
 * .Description:      .
 * .Code Status:      .Tested
 * .Reference:        .
 * .Revision History: .
 *
 *****************************************************************************/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Net.Sockets;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.Threading;

namespace VXI_11_Discovery_Tool
{
    public partial class MainForm : Form
    {
        //Constant variables
        private const int RPC_CLNT_PORT         = 9757;
        private const int RPC_SRV_PORT          = 111;
        private const int MAX_BUF_SIZE          = 255;
        private const int RECEIVE_TMO           = 2000; //in milliseconds
        private const int LINK_ID_PCKT_INDX     = 32;
        private const int LINK_ID_SIZE          = 4;
        private const int CREATE_LINK_PCKT_LEN  = 68;
        private const int DEV_WRITE_PCKT_LEN    = 72;
        private const int DEV_READ_PCKT_LEN     = 68;
        private const int DESTROY_LINK_PCKT_LEN = 48;
        private const int IDN_RESP_START_INDX   = 40;
        private const int ZERO                  = 0;
        
        //Global variables
        private int m_xid                       = 1000;


        /************************* FUNCTION HEADER ************************************
         *	Function      : MainForm
         *	Description   : Constructor for the main GUI.
         *	Procedure     :
         *	Return Value  : 
         *	
         *****************************************************************************/
        public MainForm()
        {
            InitializeComponent();
            try
            {
                labelHostNameVal.Text = Dns.GetHostName();
                IPAddress[] arrIPAddress =
                            Dns.GetHostEntry(Dns.GetHostName()).AddressList;
                labelIPAddressVal.Text = arrIPAddress[ZERO].ToString();
                buttonRefresh_Click(buttonRefresh, null);
            }
            catch (Exception Ex)
            {
                Console.Write(Ex.StackTrace);
                MessageBox.Show("IP Address of Host Machine is not obtained");
            }
        }

        /************************* FUNCTION HEADER ************************************
         *	Function      : buttonRefresh_Click
         *	Description   : This method is called when the Refresh button is clicked.
         *	Procedure     : All operations done here.
         *	Return Value  : 
         *	
         *****************************************************************************/
        private void buttonRefresh_Click(object sender, EventArgs e)
        {
            buttonRefresh.Enabled = false;
            RpcCallPacket sRpcCallPacket = new RpcCallPacket();
            //Clearing the previous data.
            listViewData.Items.Clear();
            int nFountCnt = ZERO;
            
            RpcCallPacketInit(ref sRpcCallPacket);
            IPEndPoint[] RemoteIPEndPoints = SendDiscoveryPcktGetResp(
                                                    GetBytes(sRpcCallPacket));
            if (null == RemoteIPEndPoints)
            {
                buttonRefresh.Enabled = true;
                return;
            }
            else if (ZERO == RemoteIPEndPoints.Length)
            {
                buttonRefresh.Enabled = true;
                return;
            }
            while (nFountCnt < RemoteIPEndPoints.Length)
            {
                ProcessDeviceCommn(RemoteIPEndPoints[nFountCnt]);
                nFountCnt++;
            }
        }

        /************************* FUNCTION HEADER ************************************
         *	Function      : listViewData_DoubleClick
         *	Description   : This method is called when ip address filed of the listview
         *                : control is double clicked.
         *	Procedure     : It opens the web page using that IP.
         *	Return Value  : 
         *	
         *****************************************************************************/
        private void listViewData_DoubleClick(object sender, EventArgs e)
        {
            ListView lvTemp = (ListView)sender;
            Process.Start("explorer.exe", "http://" + 
                                            lvTemp.FocusedItem.Text + "/");
        }

        /************************* FUNCTION HEADER ************************************
         *	Function      : RpcCallPacketInit
         *	Description   : This method is used to initialise the RpcCallPacket variable.
         *	Procedure     : 
         *	Return Value  : 
         *	
         *****************************************************************************/
        private void RpcCallPacketInit(ref RpcCallPacket rRpcCallPacket)
        {
            rRpcCallPacket.xid = (uint)IPAddress.HostToNetworkOrder(m_xid);
            rRpcCallPacket.type = (uint)IPAddress.HostToNetworkOrder(ZERO);
            rRpcCallPacket.cbody.rpcvers = (uint)IPAddress.HostToNetworkOrder(2);
            rRpcCallPacket.cbody.prog = (uint)IPAddress.HostToNetworkOrder(100000);
            rRpcCallPacket.cbody.vers = (uint)IPAddress.HostToNetworkOrder(2);
            rRpcCallPacket.cbody.proc = (uint)IPAddress.HostToNetworkOrder(3);
            rRpcCallPacket.cbody.cred.flavor = auth_flavor.AUTH_NULL;
            rRpcCallPacket.cbody.cred.length = (uint)IPAddress.HostToNetworkOrder(ZERO);
            rRpcCallPacket.cbody.verf.flavor = auth_flavor.AUTH_NULL;
            rRpcCallPacket.cbody.verf.length = (uint)IPAddress.HostToNetworkOrder(ZERO);
            rRpcCallPacket.prgm = (uint)IPAddress.HostToNetworkOrder(395183);
            rRpcCallPacket.vers = (uint)IPAddress.HostToNetworkOrder(1);
            rRpcCallPacket.proto = (uint)IPAddress.HostToNetworkOrder(6);
            rRpcCallPacket.port = (uint)IPAddress.HostToNetworkOrder(ZERO);
            m_xid = 1000 + (m_xid + 1)%1000;
        }

        /************************* FUNCTION HEADER ************************************
         *	Function      : RpcReplyPacketInit
         *	Description   : This method is used to initialise the RpcReplyPacket variable.
         *	Procedure     : 
         *	Return Value  : 
         *	
         *****************************************************************************/
        private void RpcReplyPacketInit(ref RpcReplyPacket sRpcReplyPacket)
        {
            sRpcReplyPacket.xid                 = ZERO;
            sRpcReplyPacket.type                = ZERO;
            sRpcReplyPacket.rbody.replystate    = ZERO;
            sRpcReplyPacket.rbody.verf.flavor   = auth_flavor.AUTH_NULL;
            sRpcReplyPacket.rbody.verf.length   = ZERO;
            sRpcReplyPacket.rbody.acceptstate   = ZERO;
            sRpcReplyPacket.port                = ZERO;
        }

        /************************* FUNCTION HEADER ************************************
         *	Function      : GetBytes
         *	Description   : This method is used to convert the RpcCallPacket variable
         *                : to byte array.
         *	Procedure     : 
         *	Return Value  : byte[]
         *	
         *****************************************************************************/
        private byte[] GetBytes(RpcCallPacket sRpcCallPacket)
        {
            int nSize = Marshal.SizeOf(sRpcCallPacket);
            IntPtr pBuff = Marshal.AllocHGlobal(nSize);
            Marshal.StructureToPtr(sRpcCallPacket, pBuff, false);
            byte[] byRawData = new byte[nSize];
            Marshal.Copy(pBuff, byRawData, ZERO, nSize);
            Marshal.FreeHGlobal(pBuff);
            return byRawData;
        }

        /************************* FUNCTION HEADER ************************************
         *	Function      : GetBytes
         *	Description   : This method is used to convert the byte array to 
         *                : RpcReplyPacket variable.
         *	Procedure     : 
         *	Return Value  : RpcReplyPacket
         *	
         *****************************************************************************/
        private RpcReplyPacket GetPacketFromBytes(byte[] byRawRespData)
        {
            RpcReplyPacket sRpcReplyPacket = new RpcReplyPacket();
            RpcReplyPacketInit(ref sRpcReplyPacket);
            int nSize = Marshal.SizeOf(sRpcReplyPacket);
            IntPtr pBuff = Marshal.AllocHGlobal(nSize);
            Marshal.Copy(byRawRespData, ZERO, pBuff, nSize);
            sRpcReplyPacket = (RpcReplyPacket)Marshal.
                            PtrToStructure(pBuff, typeof(RpcReplyPacket));
            Marshal.FreeHGlobal(pBuff);
            return sRpcReplyPacket;
        }

        /************************* FUNCTION HEADER ************************************
         *	Function      : SendDiscoveryPcktGetResp
         *	Description   : This method broadcast the RPC call packet and receive the 
         *                : response from the network devices. it also identifies the 
         *                : devices which have specified RPC program. Return their 
         *                : details.
         *	Procedure     : 
         *	Return Value  : IPEndPoint[]
         *	
         *****************************************************************************/
        IPEndPoint[] SendDiscoveryPcktGetResp(byte[] byRawData)
        {
            try
            {
                IPEndPoint RemoteIpEndPoint = new IPEndPoint(IPAddress.Broadcast,
                                                                    RPC_SRV_PORT);
                UdpClient udpClient = new UdpClient(RPC_CLNT_PORT);
                udpClient.EnableBroadcast = true;
                udpClient.Client.Blocking = true;
                udpClient.Client.EnableBroadcast = true;
                udpClient.Client.ReceiveTimeout = RECEIVE_TMO;
                udpClient.Send(byRawData, byRawData.Length, RemoteIpEndPoint);
                IPEndPoint[] tempRemoteIPEndPoints = new IPEndPoint[MAX_BUF_SIZE];
                IPEndPoint[] retRemoteIPEndPoints = null;
                int nFoundCnt = ZERO;
                while (true)
                {
                    try
                    {
                        IPEndPoint RemoteIpEndPointClnts = new IPEndPoint(
                                                                IPAddress.Any, ZERO);
                        byte[] byRawRespData = udpClient.Receive(
                                                        ref RemoteIpEndPointClnts);
                        RpcReplyPacket sRpcReplyPacket = GetPacketFromBytes(
                                                                    byRawRespData);
                        int nPort = (int)IPAddress.NetworkToHostOrder(
                                                        (int)sRpcReplyPacket.port);
                        if (ZERO != nPort)
                        {
                            tempRemoteIPEndPoints[nFoundCnt] =
                                                            RemoteIpEndPointClnts;
                            tempRemoteIPEndPoints[nFoundCnt].Port = nPort;
                            nFoundCnt++;
                        }
                    }
                    catch (Exception Ex)
                    {
                        Console.Write(Ex.StackTrace);
                        break;
                    }
                }
                if (ZERO < nFoundCnt)
                {
                    retRemoteIPEndPoints = new IPEndPoint[nFoundCnt];
                    for (int nCnt = ZERO; nCnt < nFoundCnt; nCnt++)
                    {
                        retRemoteIPEndPoints[nCnt] = tempRemoteIPEndPoints[nCnt];
                    }
                }
                udpClient.Close();
                return retRemoteIPEndPoints;
            }
            catch (Exception Ex)
            {
                Console.Write(Ex.StackTrace);
            }
            return null;
        }

        /************************* FUNCTION HEADER ************************************
         *	Function      : GetDataFromDevice
         *	Description   : This method is used to get the *IDN? response from the 
         *                : discovered devices. The each packet is constructed and 
         *                : send to each of the devices. Also reads the response 
         *                : for each packet.
         *	Procedure     : 
         *	Return Value  : bool
         *	
         *****************************************************************************/
        private bool GetDataFromDevice( IPEndPoint RemoteIPEndPoint,
                                        ref String strManufacturer,
                                        ref String strModel,
                                        ref String strSerial,
                                        ref String strVersion)
        {
            bool bRet = false;
            TcpClient tcpClient = new TcpClient();
            try
            {
                byte[] byRespBuffer = new byte[MAX_BUF_SIZE];
                tcpClient.Client.Blocking = true;
                tcpClient.Client.ReceiveTimeout = RECEIVE_TMO;
                tcpClient.Connect(RemoteIPEndPoint);
                byte[] byCreateLinkPacket = new byte[CREATE_LINK_PCKT_LEN]{	
                        0x80, 0x00, 0x00, 0x40, 0x00, 0x00, 0x03, 0xe9, 0x00, 
					    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x00, 0x06,
					    0x07, 0xaf, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00,
					    0x0a, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
					    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
					    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
					    0x07, 0xd0, 0x00, 0x00, 0x00, 0x05, 0x69, 0x6e, 0x73,
					    0x74, 0x30, 0x00, 0x00, 0x00 };
                tcpClient.Client.Send(byCreateLinkPacket);
                tcpClient.Client.Receive(byRespBuffer);
                byte[] byLinkId = new byte [LINK_ID_SIZE]{
                                        byRespBuffer[LINK_ID_PCKT_INDX],
                                        byRespBuffer[LINK_ID_PCKT_INDX + 1],
                                        byRespBuffer[LINK_ID_PCKT_INDX + 2],
                                        byRespBuffer[LINK_ID_PCKT_INDX + 3]};
                byRespBuffer.Initialize();
                
                byte[] byIdnCmdDevWritePacket = new byte[DEV_WRITE_PCKT_LEN] { 
                        0x80, 0x00, 0x00, 0x44, 0x00, 0x00, 0x03, 0xea, 0x00, 
                        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x00, 0x06, 
                        0x07, 0xaf, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 
                        0x0b, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
                        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
                        byLinkId[0], byLinkId[1], byLinkId[2], byLinkId[3], 
                        0x00, 0x00, 0x0b, 0xb8, 0x00, 0x00, 0x0b, 0xb8, 0x00, 
                        0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x06, 0x2a, 0x49, 
                        0x44, 0x4e, 0x3f, 0x0a, 0x00, 0x00 };
                tcpClient.Client.Send(byIdnCmdDevWritePacket);
                tcpClient.Client.Receive(byRespBuffer);
                byRespBuffer.Initialize();
                
                byte[] byIdnCmdDevReadPacket = new byte[DEV_READ_PCKT_LEN] { 
                        0x80, 0x00, 0x00, 0x40, 0x00, 0x00, 0x03, 0xeb, 0x00, 
                        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x00, 0x06, 
                        0x07, 0xaf, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 
                        0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
                        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
                        byLinkId[0], byLinkId[1], byLinkId[2], byLinkId[3], 
                        0x00, 0x00, 0x04, 0x00, 0x00, 0x00, 0x0b, 0xb8, 0x00, 
                        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
                        0x00, 0x00 };
                tcpClient.Client.Send(byIdnCmdDevReadPacket);
                tcpClient.Client.Receive(byRespBuffer);
                System.Text.ASCIIEncoding encTemp = new System.Text.
                                                            ASCIIEncoding();
                String strIdnResp = encTemp.GetString(
                                        byRespBuffer, 
                                        IDN_RESP_START_INDX,
                                        (byRespBuffer.Length - 
                                        IDN_RESP_START_INDX));
                byRespBuffer.Initialize();
                
                byte[] byDestroyLinkPacket = new byte[DESTROY_LINK_PCKT_LEN] { 
                        0x80, 0x00, 0x00, 0x2c, 0x00, 0x00, 0x03, 0xee, 0x00, 
                        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x00, 0x06, 
                        0x07, 0xaf, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 
                        0x17, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
                        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
                        byLinkId[0], byLinkId[1], byLinkId[2], byLinkId[3] };
                tcpClient.Client.Send(byDestroyLinkPacket);
                tcpClient.Client.Receive(byRespBuffer);
                byRespBuffer.Initialize();
                tcpClient.Close();
                char[] szSep = { ',' };
                strIdnResp.Trim();
                String[] strTempArray = strIdnResp.Split(szSep);
                if (4 == strTempArray.Length)
                {
                    strManufacturer = strTempArray[0];
                    strModel        = strTempArray[1];
                    strSerial       = strTempArray[2];
                    strVersion      = strTempArray[3]; 
                    bRet = true;
                }
            }
            catch(Exception Ex)
            {
                Console.Write(Ex.StackTrace);
                if (true == tcpClient.Connected)
                {
                    tcpClient.Close();
                }
                bRet = false;
                //MessageBox.Show("Time out");
            }
            return bRet;
        }

        /************************* FUNCTION HEADER ************************************
         *	Function      : ProcessThread
         *	Description   : This method is a thread which calls GetDataFromDevice for 
         *                : each device.
         *	Procedure     : 
         *	Return Value  : 
         *	
         *****************************************************************************/
		private void ProcessDeviceCommn(IPEndPoint RemoteIPEndPoint)
        {
            String strIPAddress = "";
            String strManufacturer = "";
            String strModel = "";
            String strSerial = "";
            String strVersion = "";
            bool bRet = false;
            strIPAddress = RemoteIPEndPoint.Address.ToString();
            bRet = GetDataFromDevice(RemoteIPEndPoint, ref strManufacturer,
                                ref strModel, ref strSerial, ref strVersion);
            ListViewItem lviTemp = new ListViewItem(new String[] 
                                        { strIPAddress, strManufacturer,
                                          strModel , strSerial,strVersion});
            listViewData.Items.Add(lviTemp);
            buttonRefresh.Enabled = true;
        }
    }
}